<?php
/**
 * Plugin Name:       Martins Free SEO Backlink Linkbuilding Network
 * Plugin URI:        https://easy-link-building.martinstools.com
 * Description:       SEO backlinks plugin for WordPress, SEO backlinks for blogs, SEO backlinks for WooCommerce. Boost your Ecommerce business sales with easy automatic link building.
 * Version:           1.2.9
 * Requires at least: 5.0
 * Requires PHP:      5.6
 * Author:            Martins Tools
 * Author URI:        https://www.martinstools.com
 * License:           GPLv2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       martins-link-network
 * Domain Path:       /languages
 */

if(!defined ('ABSPATH')) {die;} // Block direct access to the file


class martinsLinkNetworkFront 
{
        
    private $version = "1.2.9-MT";
    private $cacheFile = "";
    private $logFile = "";
    private $versionFile = "";
    private $links = [];
    private $data = "";
    private $usedKeyword = [];
    
    
    function __construct() 
    {
        // Set file locations
        $uploadDir = wp_upload_dir();        
        $this->cacheFile = $uploadDir["basedir"] . "/martinsLinkNetworkCache.txt";
        $this->logFile = $uploadDir["basedir"] . "/martinsLinkNetworkLog.txt";
        $this->versionFile = $uploadDir["basedir"] . "/martinsLinkNetworkVersion.txt";
        
        // Setup plugin
        add_action('wp', [$this, "getData"], 1000000);
        // Try adding links to content
        add_filter('the_content', [$this, "insertLinksContent"], 1000000);
        // Try adding links to comments
        add_filter('comment_text', [$this, "insertLinksContent"], 1000000);
        // Try adding links to footer
        add_action('wp_footer', [$this, "insertLinksFooter"], 1000000);
    }
     
    
    public function getData() 
    {       
        // Initiate file system
        require_once (ABSPATH . "/wp-admin/includes/file.php");
        WP_Filesystem();
        global $wp_filesystem;
        
        // Clean cache and log if version has changed
        if ($wp_filesystem->get_contents($this->versionFile) !== $this->version) {
            if (is_file($this->cacheFile)) {
                unlink($this->cacheFile);
            }
            if (is_file($this->logFile)) {
                unlink($this->logFile);
            }
            $wp_filesystem->put_contents($this->versionFile, $this->version); 
        }
        
        // Get log data
        $logData = "";
        if ($wp_filesystem->exists($this->logFile)) {
            $logData = $wp_filesystem->get_contents($this->logFile);
        }
        
        // Get data from cache if exists
        $isCache = false;        
        if ($wp_filesystem->exists($this->cacheFile) && time() - $wp_filesystem->mtime($this->cacheFile) < 24 * 3600) { // 1 day
            $this->data = $wp_filesystem->get_contents($this->cacheFile);
            $isCache = true;
        }
        
        // Get data from API if no cached data
        else {
            $result = wp_remote_post("https://linknetwork.martinstools.com/api/domains", ['timeout' => 30, 'method' => 'POST', 'body' => ["url" => get_site_url(), "version" => $this->version, "logData" => $logData]]);
            if (!isset($result->errors)) {
                $this->data = $result["body"];
                
                if (!$wp_filesystem->put_contents($this->cacheFile, $this->data)) {
                    echo("<hr style='margin:0;padding:0;height:1px;border:0;' /><div style='text-align:center;'><b>Martins Link Network plugin, could not write cache to upload folder!</b><br />Please check your folder permissions...</div>");
                }
            }
        }
        
        // Save current url to log
        if (!$isCache) {
            // Cleanup log and save only this url if new data was requested from server
            $log = [md5(get_permalink()) => get_permalink()];
        }
        else {
            $log = json_decode($logData, true);
            $log[md5(get_permalink())] = get_permalink();
        }
        $wp_filesystem->put_contents($this->logFile, json_encode($log));       
       
        // Extract links from data
        if ($this->data <> "") {
            $this->links = json_decode($this->data, true);
        }      
    }
    
    
    function insertLinksContent($content) 
    {
        // Check if we're inside the main loop in a single Post.
        if (count($this->links) > 0 && is_main_query() && in_the_loop()) {
            $i = 0;
            $extend = "";
            
            foreach($this->links as $link) {
                
                // Try match pages keywords IF pages exists (Some websites does not have any pages)
                $isMatched = false;
                if (isset($link["pages"])) {
                    shuffle($link["pages"]);
                    
                    foreach($link["pages"] as $page) {
                        foreach($page["keywords"] as $keyword) {
                            $pos = strrpos($content, " " . $keyword . " ");
                            if ($pos !== false && !in_array($keyword, $this->usedKeyword) && substr(substr($content, $pos), strpos(substr($content, $pos), "<"), 3) != "</a")
                            {
                                $content = substr_replace($content, "<a href='" . $page["url"] . "' target='_blank'>" . $keyword . "</a>", $pos+1, strlen($keyword));
                                unset($this->links[$i]);
                                $this->usedKeyword[] = $keyword;
                                $isMatched = true;
                                break 2; // Only make 1 keyword clickable
                            }
                        }
                    }
                }
                
                // Try matching main keywords
                if (!$isMatched) { 
                    foreach($link["keywords"] as $keyword) {
                        $pos = strrpos($content, " " . $keyword . " ");
                        
                        if ($pos !== false && !in_array($keyword, $this->usedKeyword) && substr(substr($content, $pos), strpos(substr($content, $pos), "<"), 3) != "</a")
                        {
                            $content = substr_replace($content, "<a href='" . $link["scheme"] . "://" . $link["domain"] . "' target='_blank'>" . $keyword . "</a>", $pos+1, strlen($keyword));
                            unset($this->links[$i]);
                            $this->usedKeyword[] = $keyword;
                            $isMatched = true;
                            break; // Only make 1 keyword clickable
                        }
                    }
                }
                
                $i++;
            }             
        }

        // Reindex links array as we might have removed some items
        $this->links = array_values($this->links);
        
        return $content;
    }
    
    
    public function insertLinksFooter() 
    {
        // Show links
        if (count($this->links) > 0) {  
            $linkStr = "";
            foreach($this->links as $link) {
                // Group main link and pages for shuffling
                $linkGroup = [];
                $mainLink = ["url" => $link["scheme"] . "://" . $link["domain"], "keywords" => $link["keywords"]];
                $linkGroup[] = $mainLink;

                if (!isset($link["pages"])) {
                    $link["pages"] = [];
                }
                foreach($link["pages"] as $page) {
                    $linkGroup[] = $page;
                } 

                // Shuffle and extract url and keyword
                shuffle($linkGroup);
                $url = $linkGroup[0]["url"];
                $keywords = $linkGroup[0]["keywords"];
                shuffle($keywords);

                // Use website name or url if no keywords
                $keyword = count($keywords) > 0 ? $keywords[0] : $link["name"];
                $keyword = $keyword ? $keyword : str_replace("http://", "", str_replace("https://", "", $url));
                
                $linkStr .= "<a style='padding:0 10px;font-size:10px;text-decoration:none;color:#777777;' href='" . $url . "'>" . mb_strtoupper($keyword) . "</a> &middot; ";
            }
            
            // Build allowed html array
            $allowedHtml = wp_kses_allowed_html();
            $allowedHtml['a'] = array();
            $allowedHtml['a']['href'] = array();
            $allowedHtml['a']['style'] = array();
            
            echo("<div style='border-top:1px solid #777777;background-color:#ffffff;padding:5px;text-align:center;' />" . wp_kses(substr($linkStr, 0, -10), $allowedHtml) . "</div>");
        }
    }
    
}


class martinsLinkNetworkAdmin 
{
    private $url = "";
    
    
    function __construct() {
        $this->url = wp_parse_url(get_site_url());
        
        add_action('admin_menu', [$this, "addMenuItems"]);
        add_filter('plugin_action_links_' . plugin_basename(__FILE__), [$this, 'add_action_links']);
    }
    
    
    function addMenuItems() {
        global $submenu;
        
        $submenu['index.php'][] = array("Martins linkbuilding", "manage_options" , "https://linknetwork.martinstools.com/admin/#/statswp/" . $this->url["host"] ); 
    }  
    
    
    function add_action_links ($links) {
        $mylinks = array(
            "<a href='https://easy-link-building.martinstools.com' target='_blank'><b>Upgrade to VIP</b></a>",
            "<a href='https://linknetwork.martinstools.com/admin/#/statswp/" . $this->url["host"] ."' target='_blank'>Dashboard</a>",
            "<a href='https://easy-link-building.martinstools.com#contact' target='_blank'>Support</a>"
        );
        
       return array_merge($mylinks, $links);
    }
}


// Start the show
if (!is_admin()) {
    $martinsLinkNetworkFront = new martinsLinkNetworkFront();   
}
else {
    $martinsLinkNetworkAdmin = new martinsLinkNetworkAdmin();
}